@@ -74,8 +74,8 @@ class TestMiniPG(unittest.TestCase):
         self.assertEqual(cur.fetchone()[0], 1)
         cur.execute("select i4 from test_basic where i2=%s", (1,))
         self.assertEqual(cur.fetchone()[0], 2)
-        cur.execute("select i2 from test_basic where s=%s", (u'あいうえお',))
-        self.assertEqual(cur.fetchone()[0], 1)
+        cur.execute("select i2, s from test_basic where s=%s", (u'あいうえお',))
+        self.assertEqual(cur.fetchone(), (1, u'あいうえお'))
         cur.execute("select count(*) from test_basic where b1=%s", (True,))
         self.assertEqual(cur.fetchone()[0], 1)
         cur.execute("select count(*) from test_basic where b1=%s", (False,))
